const fs = require('fs');
const path = require('path');
const sharp = require('sharp');
const { uploadDir, avatarDir } = require('../middleware/upload');

const uploadFiles = async (req, res) => {
  try {
    if (!req.files || req.files.length === 0) {
      return res.status(400).json({ error: 'No files uploaded' });
    }

    const uploadedFiles = req.files.map(file => ({
      filename: file.filename,
      originalName: file.originalname,
      mimetype: file.mimetype,
      size: file.size,
      path: `/uploads/${file.filename}`,
      uploadedAt: new Date().toISOString()
    }));

    res.json({
      message: 'Files uploaded successfully',
      files: uploadedFiles,
      count: uploadedFiles.length
    });
  } catch (error) {
    console.error('Upload error:', error);
    res.status(500).json({ error: 'Internal server error' });
  }
};

const uploadAvatar = async (req, res) => {
  try {
    if (!req.file) {
      return res.status(400).json({ error: 'No avatar file uploaded' });
    }

    const originalPath = req.file.path;
    const filename = req.file.filename;
    const resizedFilename = `resized-${filename}`;
    const resizedPath = path.join(avatarDir, resizedFilename);

    await sharp(originalPath)
      .resize(200, 200, {
        fit: 'cover',
        position: 'center'
      })
      .jpeg({ quality: 90 })
      .toFile(resizedPath);

    fs.unlinkSync(originalPath);

    const avatarUrl = `/avatars/${resizedFilename}`;

    res.json({
      message: 'Avatar uploaded successfully',
      avatar: {
        filename: resizedFilename,
        originalName: req.file.originalname,
        size: req.file.size,
        url: avatarUrl,
        uploadedAt: new Date().toISOString()
      }
    });
  } catch (error) {
    console.error('Avatar upload error:', error);
    res.status(500).json({ error: 'Internal server error' });
  }
};

const getFile = async (req, res) => {
  try {
    const { filename } = req.params;
    const filePath = path.join(uploadDir, filename);

    if (!fs.existsSync(filePath)) {
      return res.status(404).json({ error: 'File not found' });
    }

    const stat = fs.statSync(filePath);
    if (!stat.isFile()) {
      return res.status(404).json({ error: 'File not found' });
    }

    res.sendFile(filePath);
  } catch (error) {
    console.error('Get file error:', error);
    res.status(500).json({ error: 'Internal server error' });
  }
};

const getAvatar = async (req, res) => {
  try {
    const { filename } = req.params;
    const filePath = path.join(avatarDir, filename);

    if (!fs.existsSync(filePath)) {
      return res.status(404).json({ error: 'Avatar not found' });
    }

    const stat = fs.statSync(filePath);
    if (!stat.isFile()) {
      return res.status(404).json({ error: 'Avatar not found' });
    }

    res.sendFile(filePath);
  } catch (error) {
    console.error('Get avatar error:', error);
    res.status(500).json({ error: 'Internal server error' });
  }
};

const deleteFile = async (req, res) => {
  try {
    const { filename } = req.params;
    const filePath = path.join(uploadDir, filename);

    if (!fs.existsSync(filePath)) {
      return res.status(404).json({ error: 'File not found' });
    }

    fs.unlinkSync(filePath);

    res.json({
      message: 'File deleted successfully',
      filename: filename
    });
  } catch (error) {
    console.error('Delete file error:', error);
    res.status(500).json({ error: 'Internal server error' });
  }
};

const listFiles = async (req, res) => {
  try {
    const files = fs.readdirSync(uploadDir);
    const fileList = files.map(filename => {
      const filePath = path.join(uploadDir, filename);
      const stat = fs.statSync(filePath);
      
      return {
        filename: filename,
        size: stat.size,
        uploadedAt: stat.mtime.toISOString(),
        url: `/uploads/${filename}`
      };
    });

    res.json({
      files: fileList,
      count: fileList.length
    });
  } catch (error) {
    console.error('List files error:', error);
    res.status(500).json({ error: 'Internal server error' });
  }
};

module.exports = {
  uploadFiles,
  uploadAvatar,
  getFile,
  getAvatar,
  deleteFile,
  listFiles
};